"""GitHub API Utils"""
from typing import List
from requests import get, post, Response
from requests.auth import HTTPBasicAuth

from gaama.exceptions import DownloadError, PublishError


HEADER_ACCEPT = 'application/vnd.github+json'
API_BASE_URL = 'https://api.github.com'
UPLOADS_BASE_URL = 'https://uploads.github.com'

class GitHub:
    """GitHub"""
    def __init__(self, username: str, password: str, owner: str, repository: str) -> None:
        self.auth = HTTPBasicAuth(username, password)
        self.owner = owner
        self.repo = repository

    def create_github_release(self, tag: str) -> str:
        """create github release"""
        # TODO(omkar): allow customizations and add more details to each release
        payload = {'tag_name': tag}
        res = post(f'{API_BASE_URL}/repos/{self.owner}/{self.repo}/releases',
            auth=self.auth, headers={'Accept': HEADER_ACCEPT}, json=payload)
        if res.status_code == 201:
            result = res.json()
            return result
        raise PublishError(res.text)

    def upload_github_release_assets(self, release_id: str, file: str) -> None:
        """upload github release assets"""
        with open(file, 'rb') as reader:
            data = reader.read()
        res = post(f'{UPLOADS_BASE_URL}/repos/{self.owner}/{self.repo}/releases/{release_id}/assets?name={file}',
            auth=self.auth, headers={'Accept': HEADER_ACCEPT, 'Content-Type': 'application/octet-stream'}, data=data)
        if res.status_code != 201:
            raise PublishError(res.text)

    def get_github_release_assets(self, tag: str) -> List[dict]:
        """get github release"""
        url = f'{API_BASE_URL}/repos/{self.owner}/{self.repo}/releases/tags/{tag}'
        if tag is None:
            url = f'{API_BASE_URL}/repos/{self.owner}/{self.repo}/releases/latest'
        assets = []
        res = get(url, auth=self.auth, headers={'Accept': HEADER_ACCEPT})
        if res.status_code == 200:
            result = res.json()
            gh_assets = result['assets']
            if len(gh_assets) == 0:
                raise Exception('no release assets were found!')
            for gh_asset in gh_assets:
                assets.append({'name': gh_asset['name'], 'url': gh_asset['url']})
            return assets
        raise DownloadError(res.text)

    def download_github_release_assets(self, artifact_url: str) -> Response:
        """download github release assets"""
        res = get(artifact_url, headers={'Accept': 'application/octet-stream'},
            auth=self.auth, stream=True)
        return res



"""GaAMA"""
from datetime import date
import io
import logging
import os
from typing import List
from zipfile import ZipFile

from gaama.github import GitHub


logging.basicConfig()
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

class GaAMA:
    """GaAMA"""
    def __init__(self, username, password, owner, repository) -> None:
        self.github = GitHub(username, password, owner, repository)

    def _get_tag(self) -> str:
        """create release tag"""
        # TODO(omkar): maybe support semVer?
        return str(date.today()).replace('-', '.') # YYYY.MM.DD

    def _create_release(self, tag: str) -> str:
        """create github release"""
        if tag is None:
            tag = self._get_tag()
        res = self.github.create_github_release(tag)
        return res['id']

    def publish(self, tag: str, files: List[str], zip_files: bool = True) -> None:
        """publish model artifacts"""
        logger.info(f'publishing total {len(files)} github assets with tag: {tag}, zip: {zip_files}')
        # get github release id
        release_id = self._create_release(tag)
        # if zip is enabled, zip all the artifacts
        if zip_files:
            zip_file = f'{tag}.zip'
            with ZipFile(zip_file, 'w') as zip_writer:
                for file in files:
                    zip_writer.write(file)
            # upload the zip to github
            self.github.upload_github_release_assets(release_id, zip_file)
            # delete the zip file after upload is finished
            os.remove(zip_file)
        # upload all the files to github
        for file in files:
            self.github.upload_github_release_assets(release_id, file)

    def download(self, tag: str, path: str = '.') -> None:
        """download model artifacts"""
        logger.info(f'downloading github assets with tag: {tag}')
        # get list of file name and file download url
        artifacts = self.github.get_github_release_assets(tag)
        for artifact in artifacts:
            # download each artifact
            res = self.github.download_github_release_assets(artifact['url'])
            # unzip if the file contains a zip extension
            if '.zip' in artifact['name']:
                with ZipFile(io.BytesIO(res.content)) as zip_file:
                    zip_file.extractall(path)
            else:
                # normal download the file
                res.raise_for_status()
                with open(artifact['name'], 'wb') as writer:
                    for chunk in res.iter_content(chunk_size=1024):
                        writer.write(chunk)



import os
import subprocess
from setuptools import setup, find_packages


version = "0.2.0"
sha = subprocess.check_output(["git", "rev-parse", "HEAD"]).strip().decode("utf-8") 
package_name = "gaama"

cwd = os.path.dirname(os.path.abspath(__file__))

def write_version_file():
    version_path = os.path.join(cwd, package_name, "version.py")
    with open(version_path, "w") as f:
        f.write("__version__ = '{}'\n".format(version))
        f.write("git_version = '{}'\n".format(repr(sha)))

write_version_file()

readme = open("README.md").read()

requirements = open("requirements.txt").read().split()

setup(
    name=package_name,
    version=version,
    author="Omkar Prabhu",
    author_email="prabhuomkar@pm.me",
    url="https://github.com/prabhuomkar/bitbeast",
    description="GitHub-as-Artifactory for Model Artifacts",
    long_description=readme,
    long_description_content_type="text/markdown",
    license="Apache License 2.0",
    packages=find_packages(exclude=("examples")),
    zip_safe=True,
    install_requires=requirements,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Intended Audience :: Science/Research",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
)